/*
 * Copyright (C) 2004-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "error.h"
static int *signal_pipe;

int para_signal_init(int *spipe)
{
	signal_pipe = spipe;
	return pipe(spipe) < 0? -E_SIGNAL_PIPE : 1;
}

/*
 * just write one integer to signal pipe
 */
static void generic_signal_handler(int s)
{
	write(signal_pipe[1], &s, sizeof(int));
	//fprintf(stderr, "got sig %i, write returned %d\n", s, ret);
}

pid_t para_reap_child(void)
{
	int status;
	pid_t pid = waitpid(-1, &status, WNOHANG);

	if (pid <= 0)
		return 0;
	if (WIFEXITED(status))
		PARA_DEBUG_LOG("child %i exited. Exit status: %i\n", pid,
			WEXITSTATUS(status));
	else if (WIFSIGNALED(status))
		PARA_DEBUG_LOG("child %i was killed by signal %i\n", pid,
			WTERMSIG(status));
	else
		PARA_WARNING_LOG("child %i terminated abormally\n", pid);
	return pid;
}

void para_reap_children(void)
{
	while (para_reap_child() > 0)
		; /* nothing */
}

int para_install_sighandler(int sig)
{
	PARA_DEBUG_LOG("catching signal %d\n", sig);
	return signal(sig, &generic_signal_handler) == SIG_ERR?  -E_SIGNAL_SIG_ERR : 1;
}

/*
 * idea taken from libdaemon
 */
int para_next_signal()
{
	int s;
	ssize_t r;

	if ((r = read(signal_pipe[0], &s, sizeof(s)) == sizeof(s)) > 0) {
		PARA_DEBUG_LOG("next signal: %d\n", s);
		return s;
	}
	return r < 0 && (errno != EAGAIN)? 0 : -E_SIGNAL_READ;
}
